#
# 2D curve demo applet
#
#   version 2.0
#
# doug zongker (dougz@cs.washington.edu)
# 19 may 2000
#

proc update_tensiondisplay {{args}} {
    global g_tension
    .b2.tensionvalue config -text $g_tension
}

proc update_paramvalue {{args}} {
    global g_adaptive g_parameter
    if {$g_adaptive} {
	.b4.paramvalue config -text [format "%.4f" [expr pow(10.0,$g_parameter)]]
    } else {
	.b4.paramvalue config -text $g_parameter
    }
}

proc update_dparamvalue {{args}} {
    global g_dparam
    .b5.dparamvalue config -text $g_dparam
}

proc redraw {{args}} {
    global g_points g_endpoint g_type g_types g_control g_beziercontrol
    global g_alternate g_alternatelines g_multiplicity g_tension
    global g_adaptive g_parameter g_showcurve
    global g_decasteljau g_dparam
    set colors [list red yellow]
    set color 0
    set dotsize 6
    .c delete stuff

    if {$g_type == "bspline"} {
	.b2.single config -state normal
	.b2.double config -state normal
	.b2.triple config -state normal
    } else {
	.b2.single config -state disabled
	.b2.double config -state disabled
	.b2.triple config -state disabled
    } 

    if {$g_type == "catmull-rom" || $g_type == "loop catmull-rom"} {
	.b2.tension config -state normal
    } else {
	.b2.tension config -state disabled
    }
    
    if {$g_control} {
	# draw control polygon
	set oldp [lindex $g_points 0]
	foreach p [lrange $g_points 1 end] {
	    .c create line [lindex $oldp 0] [lindex $oldp 1] [lindex $p 0] [lindex $p 1] -width 1 -fill "#a0a0a0" -tags stuff
	    set oldp $p
	}
    }

    switch -- $g_type {
	"high order beziers" {
	    if {[llength $g_points]>0} {
		set temp [llength $g_points]
		foreach pt $g_points {
		    append temp " $pt"
		}
		set curve [list $temp]
	    } else {
		set curve {}
	    }
	}
	"cubic beziers" {
	    set curve [curvedemo::bezier3 $g_points]
	}
	"bspline" {
	    set curve [curvedemo::bspline $g_multiplicity $g_points]
	}
	"loop bspline" {
	    set curve [curvedemo::loopbspline $g_points]
	}
	"catmull-rom" {
	    set curve [curvedemo::catmullrom $g_tension $g_points]
	}
	"loop catmull-rom" {
	    set curve [curvedemo::loopcatmullrom $g_tension $g_points]
	}
	"c2 interpolating" {
	    set curve [curvedemo::c2interpolating $g_points]
	}
	"loop c2 interpolating" {
	    set curve [curvedemo::loopc2interpolating $g_points]
	}
	default {
	    set curve {}
	}
    }
    
    if {[llength $curve] > 0} {

	if {$g_decasteljau} {
	    set u [expr $g_dparam * [llength $curve]]
	    set i [expr int($u)]
	    if {$i > [llength $curve]-1} {set i [expr [llength $curve]-1]}
	    set u [expr $u-$i]
	    if {$u > 1.0} {set u 1.0}

	    set dpts [curvedemo::decasteljau $u [lindex $curve $i]]

	    foreach item [lindex $dpts 2] {
		eval .c create line $item -fill black -width 1 -tags stuff
	    }
	    set x [lindex $dpts 0]
	    set y [lindex $dpts 1]
	    .c create oval [expr $x-$dotsize] [expr $y-$dotsize] [expr $x+$dotsize] [expr $y+$dotsize] -fill red -tags [list stuff dots] -outline {}
	}
	
	if {$g_beziercontrol} {
	    foreach segment $curve {
		draw_bezier_polygon $segment
	    }
	}

	if {$g_showcurve} {
	    if {$g_alternate} {
		if {$g_alternatelines} {
		    set colors [list [list red yellow] [list blue green]]
		} else {
		    set colors [list red yellow]
		}
	    } else {
		if {$g_alternatelines} {
		    set colors [list [list red yellow]]
		} else {
		    set colors red
		}
	    }
	    
	    set i 0
	    set sum 0
	    foreach segment $curve {
		incr sum [draw_bezier_curve $segment [lindex $colors [expr $i%[llength $colors]]]]
		incr i
	    }
	    
	    .b.segments config -text "$sum line segments"
	} else {
	    .b.segments config -text ""
	}

	.c raise dots

    } else {
	.b.segments config -text ""
    }
    
    # draw control points
    set i 0
    foreach p $g_points {
	set x [lindex $p 0]
	set y [lindex $p 1]
	set id [.c create oval [expr $x-$dotsize] [expr $y-$dotsize] [expr $x+$dotsize] [expr $y+$dotsize] -fill blue -tags stuff -outline {}]
	.c bind $id <ButtonPress-1> "point_down $i %x %y"
	incr i
    }

}

proc change_subdivision {{args}} {
    global g_adaptive g_parameter g_oldparameter
    
    set foo $g_oldparameter
    set g_oldparameter $g_parameter
    if {$g_adaptive} {
	.b4.l config -text "flatness:"
	.b4.param config -from -4.0 -to 0 -res 0.001
	set g_parameter $foo
    } else {
	.b4.l config -text "subdivisions:"
	.b4.param config -from 1 -to 50 -res 1
	set g_parameter $foo
    }
    redraw
}
    

proc draw_bezier_polygon {p} {
    set dotsize 4
    
    set size [lindex $p 0]
    if {[llength $p] != ($size*2+1)} return

    if {$size >= 2} {
	eval .c create line [lrange $p 1 end] -fill gray30 -width 2 -tags stuff
    }

    for {set i 0} {$i < $size} {incr i} {
	set x [lindex $p [expr 1+$i*2]]
	set y [lindex $p [expr 2+$i*2]]
	.c create oval [expr $x-$dotsize] [expr $y-$dotsize] [expr $x+$dotsize] [expr $y+$dotsize] -fill purple -tags [list dots stuff] -outline {}
    }
}

proc draw_bezier_curve {p colors} {
    global g_adaptive g_parameter
    
    set size [lindex $p 0]
    if {[llength $p] != ($size*2+1)} return

    if {$g_adaptive} {
	set value [expr pow(10.0,$g_parameter)]
	set pts [curvedemo::bezier_curve $g_adaptive $value $p]
    } else {
	set pts [curvedemo::bezier_curve $g_adaptive $g_parameter $p]
    }
    
    if {[llength $pts] >= 4} {
	if {[llength $colors]==1} {
	    eval .c create line $pts -fill $colors -width 3 -tags stuff
	} else {
	    set segs [expr ([llength $pts]/2)-1]
	    for {set i 0} {$i < $segs} {incr i} {
		eval .c create line [lrange $pts [expr $i*2] [expr $i*2+3]] -fill [lindex $colors [expr $i%[llength $colors]]] -width 3 -tags stuff
	    }
	}
    }

    return [expr [llength $pts]/2-1]
}

proc clear_points {} {
    global g_points 
    
    .c delete stuff
    set g_points {}
}

proc point_down {num x y} {
    global g_points
    bind .c <Motion> "point_move $num %x %y"
    bind .c <ButtonRelease-1> "point_up"
}

proc point_move {num x y} {
    global g_points
    set g_points [concat [lrange $g_points 0 [expr $num-1]] \
	    [list [list [.c canvasx $x] [.c canvasy $y]]] \
	    [lrange $g_points [expr $num+1] end]]
    redraw
}

proc point_up {} {
    bind .c <Motion> ""
    bind .c <ButtonRelease-3> ""
}

proc create_point {x y} {
    global g_points
    
    set x [.c canvasx $x]
    set y [.c canvasy $y]

    lappend g_points [list $x $y]
    redraw
}

if {[catch {load curvedemo}]} {
    if {[catch {load debug/curvedemo}]} {
	error "can't find curvedemo.dll!"
    }
}

set g_points  {}
set g_tension 0.5
set g_control 1
set g_beziercontrol 0
set g_multiplicity 1
set g_alternate 0
set g_alternatelines 0
set g_type "high order beziers"
set g_adaptive false
set g_parameter 16
set g_oldparameter -3.0
set g_showcurve 1
set g_decasteljau 0
set g_dparam 0.5

canvas .c -width 800 -height 600
.c create rect -10 -10 2000 2000 -fill [.c cget -bg] -tags bg
pack .c -expand 1 -fill both
.c bind bg <1> [list create_point %x %y]

frame .b
tk_optionMenu .b.types g_type {high order beziers} {cubic beziers} bspline {loop bspline} catmull-rom {loop catmull-rom} {c2 interpolating} {loop c2 interpolating} 
button .b.clear -text "clear" -command clear_points
button .b.quit -text "quit" -command exit
label .b.segments
pack .b.types .b.clear .b.quit -side left
pack .b.segments -side right

frame .b2
radiobutton .b2.single -text "single" -variable g_multiplicity -value 1 -command redraw
radiobutton .b2.double -text "double" -variable g_multiplicity -value 2 -command redraw
radiobutton .b2.triple -text "triple" -variable g_multiplicity -value 3 -command redraw
label .b2.l -text "tension:"
label .b2.tensionvalue -width 10 -justify center
scale .b2.tension -from 0.0 -to 10.0 -res 0.1 -showvalue false -variable g_tension -command redraw -orient h -state disabled
pack .b2.tension -expand 1 -fill x -side right
pack .b2.single .b2.double .b2.triple .b2.l .b2.tensionvalue -side left

frame .b3
checkbutton .b3.showcurve -text "show curve" -variable g_showcurve -command redraw
checkbutton .b3.control -text "spline control" -variable g_control -command redraw
checkbutton .b3.beziercontrol -text "bezier control" -variable g_beziercontrol -command redraw
checkbutton .b3.alternate -text "alternate beziers" -variable g_alternate -command redraw
checkbutton .b3.alternatelines -text "alternate lines" -variable g_alternatelines -command redraw
pack .b3.showcurve .b3.control .b3.beziercontrol .b3.alternate .b3.alternatelines -side left

frame .b4
checkbutton .b4.adaptive -text "adaptive subdivision" -variable g_adaptive -command change_subdivision
label .b4.l -text "subdivisions:" -anchor e -width 20
label .b4.paramvalue -width 10
scale .b4.param -from 1 -to 50 -res 1 -showvalue false -variable g_parameter -command redraw -orient h
pack .b4.param -expand 1 -fill x -side right
pack .b4.adaptive .b4.l .b4.paramvalue -side left

frame .b5
checkbutton .b5.decasteljau -text "decasteljau" -variable g_decasteljau -command redraw
label .b5.l -text "parameter:" -anchor e -width 20
label .b5.dparamvalue -width 10
scale .b5.dparam -from 0.0 -to 1.0 -res 0.0001 -showvalue false -variable g_dparam -command redraw -orient h
pack .b5.dparam -expand 1 -fill x -side right
pack .b5.decasteljau .b5.l .b5.dparamvalue -side left

pack .b .b2 .b3 .b4 .b5 -fill x

trace variable g_type w redraw
trace variable g_tension w update_tensiondisplay
trace variable g_parameter w update_paramvalue
trace variable g_adaptive w update_paramvalue
trace variable g_dparam w update_dparamvalue

update_tensiondisplay
update_paramvalue
update_dparamvalue



    

    
    
    
