#
# deCasteljau's algorithm demo applet
#
# doug zongker (dougz@cs.washington.edu)
# 17 november 1999
#

set g_points  {}
set g_u 0.0

canvas .c -width 800 -height 600
pack .c -expand 1 -fill both
bind .c <1> [list create_point %x %y]

frame .f

button .f.c -text clear -command clear_points
button .f.h -text half -command {set g_u 0.5; redraw}
button .f.s -text scan -command scan_u
button .f.v -text curve -command draw_curve

scale .f.u -from 0.0 -to 1.0 -res 0.001 -orient h -variable g_u -command redraw
button .f.q -text quit -command exit

pack .f.c .f.h .f.s .f.v -side left
pack .f.u -expand 1 -fill x -side left
pack .f.q -side left

pack .f -fill x

proc scan_u {} {
    for {set u 0.0} {$u <= 1.0} {set u [expr $u+0.01]} {
	.f.u set $u
	update
    }
}

proc clear_points {} {
    global g_points g_u g_save
    
    set g_u 0.0
    .c delete all
    set g_points {}
    unset g_save
}

proc draw_curve {} {
    global g_save
    set coords {}
    foreach p [lsort -real [array names g_save]] {
	append coords "$g_save($p) "
    }
    .c delete dots
    .c delete const
    eval .c create line $coords -fill red -width 3
}

proc redraw {{args}} {
    global g_points g_u g_save
    set dotsize 5
    .c delete stuff
    set oldp [lindex $g_points 0]
    foreach p [lrange $g_points 1 end] {
	.c create line [lindex $oldp 0] [lindex $oldp 1] [lindex $p 0] [lindex $p 1] -width 3 -fill "#a0a0a0" -tags stuff
	set oldp $p
    }
    foreach p $g_points {
	set x [lindex $p 0]
	set y [lindex $p 1]
	.c create oval [expr $x-$dotsize] [expr $y-$dotsize] [expr $x+$dotsize] [expr $y+$dotsize] -fill blue -tags stuff -outline {}
    }
    
    set foo $g_points
    set step 0
    while {[llength $foo]>1} {
	set newfoo {}
	set oldp [lindex $foo 0]
	foreach p [lrange $foo 1 end] {
	    if {$step} {
		.c create line [lindex $oldp 0] [lindex $oldp 1] [lindex $p 0] [lindex $p 1] -tags [list stuff const]
	    }
	    lappend newfoo [list \
		    [expr [lindex $oldp 0]*(1.0-$g_u)+[lindex $p 0]*$g_u] \
		    [expr [lindex $oldp 1]*(1.0-$g_u)+[lindex $p 1]*$g_u]]
	    set oldp $p
	    
	}
	incr step
	set foo $newfoo
    }
    if {[llength $foo]==1} {
	set x [lindex [lindex $foo 0] 0]
	set y [lindex [lindex $foo 0] 1]
	.c create oval [expr $x-$dotsize] [expr $y-$dotsize] [expr $x+$dotsize] [expr $y+$dotsize] -fill red -outline {} -tag dots
	set g_save($g_u) [lindex $foo 0]
    }
	    
}

proc create_point {x y} {
    global g_points
    
    set x [.c canvasx $x]
    set y [.c canvasy $y]

    lappend g_points [list $x $y]
    redraw
}


    

    
    
    
